#include <stdlib.h>
#include <string.h>
#include "CFile.h"

CFile file;

void CFile::printError(const char* error, const char* fileName)
{
    printf("%s : %s\n", fileName, error);
    exit(EXIT_FAILURE);
}

void CFile::printErrorAndClose(const char* error, SFile* f)
{
    printf("%s : %s\n", f->fileName, error);
    close(f);
    exit(EXIT_FAILURE);
}

CFile::SFile* CFile::open(const char* fileName, const char* mode)
{
    FILE*   f = fopen(fileName, mode);

    if (f == NULL)
        printError("Error while opening file", fileName);

    SFile* fmem = new SFile;

    if (fmem == NULL)
        printError("Error while opening file", fileName);

    fmem->file = f;
    fmem->fileName = (char*)malloc(strlen(fileName) + 1);

    if (fmem->fileName == NULL)
        printError("Error while opening file", fileName);

    strcpy(fmem->fileName, fileName);
    return fmem;
}

void CFile::close(SFile* f)
{
    int ret = fclose(f->file);

    if (ret == EOF)
        printError("Error while closing file", f->fileName);

    free(f->fileName);
    free(f);
}

void CFile::seek(SFile* f, long int offset, int origin)
{
    int ret = fseek(f->file, offset, origin);

    if (ret != 0)
        printErrorAndClose("Error while seeking file", f);
}

long int CFile::tell(SFile* f)
{
    long int ret = ftell(f->file);

    if (ret != -1L)
        printErrorAndClose("Error while telling file", f);

    return ret;
}

void CFile::read(SFile* f, void* dest, int nbBytes)
{
    int ret = fread(dest, 1, nbBytes, f->file);

    if (ret != nbBytes)
        printErrorAndClose("Error while reading file", f);
}

Uint8 CFile::readU8(SFile* f)
{
    Uint8 value;
    read(f, &value, 1);
    return value;
}

Sint8 CFile::readS8(SFile* f)
{
    Sint8 value;
    read(f, &value, 1);
    return value;
}

Uint16 CFile::readU16(SFile* f)
{
    Uint16 value;
    read(f, &value, 2);
    return value;
}

Sint16 CFile::readS16(SFile* f)
{
    Sint16 value;
    read(f, &value, 2);
    return value;
}

Uint32 CFile::readU32(SFile* f)
{
    Uint32 value;
    read(f, &value, 4);
    return value;
}

Sint32 CFile::readS32(SFile* f)
{
    Sint32 value;
    read(f, &value, 4);
    return value;
}

Uint64 CFile::readU64(SFile* f)
{
    Uint64 value;
    read(f, &value, 8);
    return value;
}

Sint64 CFile::readS64(SFile* f)
{
    Sint64 value;
    read(f, &value, 8);
    return value;
}

float CFile::readFloat(SFile* f)
{
    float value;
    read(f, &value, 4);
    return value;
}

double CFile::readDouble(SFile* f)
{
    double value;
    read(f, &value, 8);
    return value;
}

void CFile::write(SFile* f, const void* src, int nbBytes)
{
    int ret = fwrite(src, 1, nbBytes, f->file);

    if (ret != nbBytes)
        printErrorAndClose("Error while writing file", f);
}

void CFile::writeU8(SFile* f, const Uint8 value)
{
    write(f, &value, 1);
}

void CFile::writeS8(SFile* f, const Sint8 value)
{
    write(f, &value, 1);
}

void CFile::writeU16(SFile* f, const Uint16 value)
{
    write(f, &value, 2);
}

void CFile::writeS16(SFile* f, const Sint16 value)
{
    write(f, &value, 2);
}

void CFile::writeU32(SFile* f, const Uint32 value)
{
    write(f, &value, 4);
}

void CFile::writeS32(SFile* f, const Sint32 value)
{
    write(f, &value, 4);
}

void CFile::writeU64(SFile* f, const Uint64 value)
{
    write(f, &value, 8);
}

void CFile::writeS64(SFile* f, const Sint64 value)
{
    write(f, &value, 8);
}

void CFile::writeFloat(SFile* f, const float value)
{
    write(f, &value, 4);
}

void CFile::writeDouble(SFile* f, const double value)
{
    write(f, &value, 8);
}
